# Generated by Django 4.2.19 on 2025-03-30 14:07

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0011_alter_characternote_options_alter_skillcheck_options_and_more'),
    ]

    operations = [
        migrations.RenameField(
            model_name='approvedskills',
            old_name='hull',
            new_name='ship',
        ),
        migrations.AlterField(
            model_name='approvedimplantset',
            name='set',
            field=models.CharField(choices=[('Amulet', 'Amulet'), ('Ascendancy ', 'Ascendancy '), ('None', 'None')], default='Amulet', max_length=16, verbose_name='Set Type'),
        ),
    ]



from enum import Enum

from corptools.models import EveItemType
from ninja import NinjaAPI, Schema

from django.db import transaction
from django.http import Http404
from django.utils import timezone

from allianceauth.eveonline.models import EveCharacter
from allianceauth.framework.api.user import get_all_characters_from_user
from allianceauth.services.hooks import get_extension_logger

from incursions.api.schema import CharacterSchema, HullSchema
from incursions.models.waitlist import (
    ActiveFleet, Fleet, FleetSquad, WaitlistCategory,
)
from incursions.providers import (
    get_characters_character_id_fleet, kick_all_fleet_members,
)


class RoleEnum(str, Enum):
    FC = "fleet_commander"
    SC = "squad_commander"
    WC = "squad_member"
    MEMBER = "wing_commander"


class FleetStatusFleetSchema(Schema):
    id: int
    boss: CharacterSchema


class FleetStatusResponse(Schema):
    fleets: list[FleetStatusFleetSchema] | None


class FleetInfoSquadSchema(Schema):
    id: int
    name: str


class FleetInfoWingSchema(Schema):
    id: int
    name: str
    squads: list[FleetInfoSquadSchema]


class FleetInfoResponse(Schema):
    fleet_id: int
    wings: list[FleetInfoWingSchema]


class FleetMemberSchema(Schema):
    character_id: int
    character_name: str | None = None
    ship: HullSchema
    role: RoleEnum


class FleetCompSquadMembersSchema(Schema):
    id: int
    name: str
    members: list[FleetMemberSchema]
    boss: FleetMemberSchema | None = None


class FleetCompWingSchema(Schema):
    id: int
    name: str
    squads: list[FleetCompSquadMembersSchema]
    boss: FleetMemberSchema | None = None


class FleetCompResponse(Schema):
    id: int | None
    wings: list[FleetCompWingSchema] | None
    members: list[FleetMemberSchema] | None
    boss: FleetMemberSchema | None = None


class FleetMembersMemberSchema(Schema):
    character_id: int
    character_name: str | None = None
    ship: HullSchema
    wl_category: str | None = None
    category: str | None = None
    role: RoleEnum


class FleetMembersResponse(Schema):
    members: list[FleetMembersMemberSchema]


class RegisterRequest(Schema):
    character_id: int
    fleet_id: int
    assignments: dict[str, tuple[int, int]]


logger = get_extension_logger(__name__)
api = NinjaAPI()


def setup(api: NinjaAPI) -> None:
    FleetAPIEndpoints(api)


class FleetAPIEndpoints:

    tags = ["Fleets"]

    def __init__(self, api: NinjaAPI) -> None:

        @api.get("/fleet/status", response={200: FleetStatusResponse, 403: dict}, tags=self.tags)
        def fleet_status(request) -> tuple[int, dict]:
            # Called regularly
            if not request.user.has_perm("incursions.basic_waitlist"):
                logger.warning(f"User {request.user} denied access to fleet status")
                return 403, {"error": "Permission denied"}

            try:
                active_fleet = ActiveFleet.get_solo()
                fleet = active_fleet.fleet
                if fleet is None:
                    raise ActiveFleet.DoesNotExist
                logger.info(f"User {request.user} retrieved active fleet {fleet.pk if fleet else None}")
                return 200, FleetStatusResponse(fleets=[fleet])
            except (ActiveFleet.DoesNotExist, Fleet.DoesNotExist):
                logger.info(f"No active fleet for user {request.user}")
                return 200, FleetStatusResponse(fleets=None)

        @api.get("/fleet/info", response={200: FleetInfoResponse, 403: dict}, tags=self.tags)
        def fleet_info(request, character_id: int) -> tuple[int, dict]:
            # Called only by Register, loads extra DB info to then register a fleet later
            if not request.user.has_perm("incursions.waitlist_manage_waitlist"):
                logger.warning(f"User {request.user} denied access to fleet info")
                return 403, {"error": "Permission denied"}

            # ARIEL check if request.body.character_id is owned by request.user
            # Recheck this
            if EveCharacter.objects.get(character_id=character_id) not in get_all_characters_from_user(request.user):
                logger.warning(f"User {request.user} attempted to load info on a fleet with different character ID")
                return 403, {"error": "Permission denied"}

            esi_fleet = get_characters_character_id_fleet(character_id=character_id)

            if not esi_fleet:
                logger.error(f"Fleet not found for character {character_id}")
                raise Http404("Fleet not found")

            try:
                boss = EveCharacter.objects.get(character_id=character_id)
            except EveCharacter.DoesNotExist:
                boss = EveCharacter.objects.create_character(character_id=character_id)

            fleet, created = Fleet.objects.update_or_create(
                pk=esi_fleet.get("fleet_id"),
                defaults={
                    "boss": boss,
                    "open": True,
                    "closed": None,
                    "is_updating": True,
                    "last_updated": timezone.now(),
                }
            )

            fleet_wings = fleet.get_fleets_fleet_id_wings()
            wings = [
                FleetInfoWingSchema(
                    id=wing.get("id"),
                    name=wing.get("name"),
                    squads=[FleetInfoSquadSchema(id=s.get("id"), name=s.get("name")) for s in wing.get("squads", [])],
                )
                for wing in fleet_wings
            ]
            logger.info(f"Fleet info fetched for fleet {fleet.pk} by user {request.user}")
            return FleetInfoResponse(fleet_id=fleet.pk, wings=wings)

        @api.get("/fleet/fleetcomp", response={200: FleetCompResponse, 403: dict}, tags=self.tags)
        # FC FleetMembers (yes shitty naming)
        # Used for the full wing/squad/ship breakdown of the fleet
        #
        # Character ID is not used currently, as we only have one fleet at a time
        # Will be used to distinguish between multiple fleets in the future
        def fleet_composition(request, character_id: int) -> tuple[int, dict]:
            if not request.user.has_perm("incursions.waitlist_fleet_view"):
                logger.warning(f"User {request.user} denied access to fleet composition")
                return 403, {"error": "Permission denied"}

            try:
                active_fleet = ActiveFleet.get_solo()
                fleet = active_fleet.fleet
                if fleet is None:
                    raise ActiveFleet.DoesNotExist
                logger.info(f"User {request.user} retrieved active fleet {fleet.pk if fleet else None}")
            except (ActiveFleet.DoesNotExist, Fleet.DoesNotExist):
                logger.info(f"No active fleet for user {request.user}")
                return 404, {"error": "No active fleet found"}

            fleet_members = fleet.get_fleet_members()
            fleet_wings = fleet.get_fleets_fleet_id_wings()
            wings: list[FleetCompWingSchema] = []
            members: list[FleetMemberSchema] = []
            fleet_boss: FleetMemberSchema | None = None

            for wing in fleet_wings:
                squads: list[FleetCompSquadMembersSchema] = []
                wing_boss: FleetMemberSchema | None = None
                for squad in wing.get("squads", []):
                    squad_members: list[FleetMemberSchema] = []
                    squad_boss: FleetMemberSchema | None = None
                    for fm in fleet_members:  # For Each Wing+Squad, Loop through all fleet members and act on match
                        if fm.get("wing_id") == wing.get("id") and fm.get("squad_id") == squad.get("id"):
                            try:
                                character = EveCharacter.objects.only("character_name").get(character_id=fm.get("character_id"))
                                ship = EveItemType.objects.only("name").get(type_id=fm.get("ship_type_id"))
                            except EveCharacter.DoesNotExist:
                                EveCharacter.objects.create_character(character_id=fm.get("character_id"))
                                return 503, {"error": "Unknown Character  found, please try again later."}
                            except EveItemType.DoesNotExist:
                                EveItemType.objects.get_or_create_from_esi(fm.get("ship_type_id"))
                                return 503, {"error": "Unknown Ship found, please try again later"}
                            if fm.get("role") == "fleet_commander":
                                fleet_boss = FleetMemberSchema(character_id=fm.get("character_id"), character_name=character.character_name, ship=HullSchema(id=ship.pk, name=ship.name), role=fm.get("role"))
                            elif fm.get("role") == "wing_commander":
                                wing_boss = FleetMemberSchema(character_id=fm.get("character_id"), character_name=character.character_name, ship=HullSchema(id=ship.pk, name=ship.name), role=fm.get("role"),)
                            elif fm.get("role") == "squad_commander":
                                squad_boss = FleetMemberSchema(character_id=fm.get("character_id"), character_name=character.character_name, ship=HullSchema(id=ship.pk, name=ship.name), role=fm.get("role"),)
                                squad_members.append(FleetMemberSchema(character_id=fm.get("character_id"), character_name=character.character_name, ship=HullSchema(id=ship.pk, name=ship.name), role=fm.get("role"),))
                            else:
                                squad_members.append(FleetMemberSchema(character_id=fm.get("character_id"), character_name=character.character_name, ship=HullSchema(id=ship.pk, name=ship.name), role=fm.get("role"),))
                            members.append(FleetMemberSchema(  # Always append everyone to the master members list, used in frontend to reduce code
                                character_id=fm.get("character_id"),
                                name=character.character_name,
                                ship=HullSchema(id=ship.pk, name=ship.name),
                                role=fm.get("role"),
                            ))
                    squads.append(FleetCompSquadMembersSchema(id=squad.get("id"),name=squad.get("name"), members=squad_members, boss=squad_boss if squad_boss else None))
                wings.append(FleetCompWingSchema(id=wing.get("id"), name=wing.get("name"), squads=squads, boss=wing_boss if wing_boss else None))

            logger.info(f"Fleet composition fetched for fleet {fleet.pk} by user {request.user}")
            return 200, FleetCompResponse(wings=wings, id=fleet.pk, member=members, boss=fleet_boss if fleet_boss else None, members=members),

        @api.get("/fleet/members", response=FleetMembersResponse, tags=self.tags)
        # Waitlist useFleetComposition
        # Used for Waitlist Category Headers, simple fetch, number of ships, etc
        #
        # Character ID is not used currently, as we only have one fleet at a time
        # Will be used to distinguish between multiple fleets in the future
        def fleet_members(request, character_id: int) -> tuple[int, dict]:
            if not request.user.has_perm("incursions.waitlist_fleet_view"):
                logger.warning(f"User {request.user} denied access to fleet members")
                return 403, {"error": "Permission denied"}

            try:
                active_fleet = ActiveFleet.get_solo()
                fleet = active_fleet.fleet
                if fleet is None:
                    raise ActiveFleet.DoesNotExist
                logger.info(f"User {request.user} retrieved active fleet {fleet.pk if fleet else None}")
            except (ActiveFleet.DoesNotExist, Fleet.DoesNotExist):
                logger.info(f"No active fleet for user {request.user}")
                return 404, {"error": "No active fleet found"}

            fleet_members = fleet.get_fleet_members()
            wings = {w.get("id"): w.get("name") for w in fleet.get_fleets_fleet_id_wings()}
            members: list[FleetMembersMemberSchema] = []

            for fm in fleet_members:
                try:
                    char = EveCharacter.objects.only("character_name").get(character_id=fm.get("character_id"))
                    ship = EveItemType.objects.only("name").get(type_id=fm.get("ship_type_id"))
                    squad = FleetSquad.objects.only("category").get(squad_id=fm.get("squad_id"))
                except FleetSquad.DoesNotExist:
                    continue
                # These are intentionally just excepts, not get_or_create, i want this api to be performant for EVE reasons, but failing is fine
                except EveCharacter.DoesNotExist:
                    EveCharacter.objects.create_character(character_id=fm.get("character_id"))
                    return 503, {"error": "Unknown Character  found, please try again later."}
                except EveItemType.DoesNotExist:
                    EveItemType.objects.get_or_create_from_esi(fm.get("ship_type_id"))
                    return 503, {"error": "Unknown Ship found, please try again later"}

                members.append(FleetMembersMemberSchema(
                    character_id=fm.get("character_id"),
                    character_name=char.character_name,
                    ship=HullSchema(id=ship.pk, name=ship.name),
                    wl_category=squad.category.name,
                    category=f"{wings.get(fm.get('wing_id'))} - {fm.get('squad_name')}",
                    role=fm.get("role"),
                ))

            logger.info(f"Fleet members listed for fleet {fleet.pk} by user {request.user}")
            return FleetMembersResponse(members=members)

        @api.post("/fleet/register", tags=self.tags)
        def register_fleet(request, body: RegisterRequest) -> tuple[int, dict]:
            if not request.user.has_perm("incursions.waitlist_manage_waitlist"):
                logger.warning(f"User {request.user} denied fleet registration")
                return 403, {"error": "Permission denied"}

            # ARIEL check if request.body.character_id is owned by request.user
            # Recheck this
            if EveCharacter.objects.get(character_id=body.character_id) not in get_all_characters_from_user(request.user):
                logger.warning(f"User {request.user} attempted to register fleet with different character ID")
                return 403, {"error": "Permission denied"}

            with transaction.atomic():
                fleet, _ = Fleet.objects.select_for_update().update_or_create(
                    pk=body.fleet_id,
                    defaults={
                        "boss": EveCharacter.objects.get(character_id=body.character_id),
                        "open": True,
                        "closed": None,
                        "is_updating": True,
                        "last_updated": timezone.now(),
                    },
                )

                active_fleet = ActiveFleet.get_solo()
                active_fleet.fleet = fleet
                active_fleet.save(update_fields=["fleet"])

                for category, (wing_id, squad_id) in body.assignments.items():
                    cat = WaitlistCategory.objects.get(pk=category)
                    FleetSquad.objects.update_or_create(
                        fleet=fleet,
                        category=cat,
                        defaults={"wing_id": wing_id, "squad_id": squad_id},
                    )

            logger.info(f"Fleet {fleet.pk} registered and squads assigned by user {request.user}")
            return "OK"

        @api.post("/fleet/close", response={200: str, 403: dict, 404: dict}, tags=self.tags)
        def close_fleet(request, boss_character_id: int) -> tuple[int, dict]:
            if not request.user.has_perm("incursions.waitlist_manage_waitlist"):
                logger.warning(f"User {request.user} denied closing fleet")
                return 403, {"error": "Permission denied"}

            active_fleet = ActiveFleet.get_solo().fleet
            if active_fleet is None:
                logger.warning(f"User {request.user} attempted to close a fleet but no active fleet was found")
                return 404, {"error": "No active fleet found"}

            active_fleet.open = False
            active_fleet.closed = timezone.now()
            active_fleet.is_updating = False
            active_fleet.last_updated = timezone.now()
            active_fleet.save(update_fields=["open", "closed", "is_updating", "last_updated"])

            count = kick_all_fleet_members(boss_character_id=active_fleet.boss.character_id, fleet_id=active_fleet.pk)
            logger.info(f"User {request.user} closed fleet {active_fleet.pk} and kicked {count} members")
            return 200, f"Kicked {count} Fleet Members"



# Generated by Django 4.2.19 on 2025-03-30 11:44

import django.core.validators
import django.db.models.deletion
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('eveonline', '0017_alliance_and_corp_names_are_not_unique'),
        ('incursions', '0010_activefleet_announcement_approvedfitting_and_more'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='characternote',
            options={'default_permissions': (), 'ordering': ['-logged_at'], 'verbose_name': 'Waitlist - Character Note', 'verbose_name_plural': 'Waitlist - Character Notes'},
        ),
        migrations.AlterModelOptions(
            name='skillcheck',
            options={'default_permissions': (), 'verbose_name': 'Waitlist - Skill Check', 'verbose_name_plural': 'Waitlist - Skill Checks'},
        ),
        migrations.AlterField(
            model_name='announcement',
            name='created_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Created At'),
        ),
        migrations.AlterField(
            model_name='announcement',
            name='created_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='eveonline.evecharacter', verbose_name='Created By'),
        ),
        migrations.AlterField(
            model_name='announcement',
            name='is_alert',
            field=models.BooleanField(default=False, verbose_name='Is Alert'),
        ),
        migrations.AlterField(
            model_name='announcement',
            name='message',
            field=models.TextField(verbose_name='Message'),
        ),
        migrations.AlterField(
            model_name='announcement',
            name='pages',
            field=models.TextField(blank=True, null=True, verbose_name='Pages'),
        ),
        migrations.AlterField(
            model_name='announcement',
            name='revoked_at',
            field=models.DateTimeField(blank=True, default=None, null=True, verbose_name='Revoked At'),
        ),
        migrations.AlterField(
            model_name='announcement',
            name='revoked_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='eveonline.evecharacter', verbose_name='Revoked By'),
        ),
        migrations.AlterField(
            model_name='approvedfitting',
            name='description',
            field=models.CharField(blank=True, max_length=255, null=True, verbose_name='Description'),
        ),
        migrations.AlterField(
            model_name='approvedfitting',
            name='dna',
            field=models.TextField(max_length=1024, unique=True, verbose_name='DNA String'),
        ),
        migrations.AlterField(
            model_name='approvedfitting',
            name='implants',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='incursions.approvedimplantset', verbose_name='Implant Set'),
        ),
        migrations.AlterField(
            model_name='approvedfitting',
            name='name',
            field=models.CharField(max_length=25, unique=True, verbose_name='Fitting Name'),
        ),
        migrations.AlterField(
            model_name='approvedfitting',
            name='tier',
            field=models.CharField(choices=[('Basic', 'Sponge'), ('Mainline', 'Mainline'), ('Alt', 'Alt'), ('Other', 'Other')], default='Mainline', max_length=16, verbose_name='Tier'),
        ),
        migrations.AlterField(
            model_name='approvedimplantset',
            name='implants',
            field=models.CharField(max_length=255, unique=True, verbose_name='Implant List'),
        ),
        migrations.AlterField(
            model_name='approvedimplantset',
            name='name',
            field=models.CharField(max_length=255, verbose_name='Set Name'),
        ),
        migrations.AlterField(
            model_name='approvedimplantset',
            name='set',
            field=models.CharField(choices=[('Amulet', 'Amulet'), ('Ascendency', 'Ascendency'), ('None', 'None')], default='Amulet', max_length=16, verbose_name='Set Type'),
        ),
        migrations.AlterField(
            model_name='badge',
            name='exclude_badge',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='excluded_by', to='incursions.badge', verbose_name='Mutually Exclusive With'),
        ),
        migrations.AlterField(
            model_name='badge',
            name='name',
            field=models.CharField(max_length=64, unique=True, verbose_name='Badge Name'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='entity_alliance',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.eveallianceinfo', verbose_name='Alliance'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='entity_character',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='entity_corporation',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecorporationinfo', verbose_name='Corporation'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='entity_type',
            field=models.CharField(choices=[('Character', 'Character'), ('Corporation', 'Corporation'), ('Alliance', 'Alliance')], default='Character', max_length=12, verbose_name='Entity Type'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='issued_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Issued At'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='issued_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='eveonline.evecharacter', verbose_name='Issued By'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='public_reason',
            field=models.CharField(blank=True, max_length=512, null=True, verbose_name='Public Reason'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='reason',
            field=models.CharField(max_length=512, verbose_name='Internal Reason'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='revoked_at',
            field=models.DateTimeField(blank=True, default=None, null=True, verbose_name='Revoked At'),
        ),
        migrations.AlterField(
            model_name='ban',
            name='revoked_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='eveonline.evecharacter', verbose_name='Revoked By'),
        ),
        migrations.AlterField(
            model_name='characterbadges',
            name='badge',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.badge', verbose_name='Badge'),
        ),
        migrations.AlterField(
            model_name='characterbadges',
            name='character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='characterbadges',
            name='granted_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Granted At'),
        ),
        migrations.AlterField(
            model_name='characterbadges',
            name='granted_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='eveonline.evecharacter', verbose_name='Granted By'),
        ),
        migrations.AlterField(
            model_name='characternote',
            name='author',
            field=models.ForeignKey(null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='eveonline.evecharacter', verbose_name='Author'),
        ),
        migrations.AlterField(
            model_name='characternote',
            name='character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='characternote',
            name='logged_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Logged At'),
        ),
        migrations.AlterField(
            model_name='characternote',
            name='note',
            field=models.TextField(verbose_name='Note'),
        ),
        migrations.AlterField(
            model_name='characterroles',
            name='character',
            field=models.OneToOneField(on_delete=django.db.models.deletion.CASCADE, primary_key=True, serialize=False, to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='characterroles',
            name='granted_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Granted At'),
        ),
        migrations.AlterField(
            model_name='characterroles',
            name='granted_by',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.SET_NULL, related_name='+', to='eveonline.evecharacter', verbose_name='Granted By'),
        ),
        migrations.AlterField(
            model_name='fitting',
            name='dna',
            field=models.TextField(max_length=1024, unique=True, verbose_name='DNA String'),
        ),
        migrations.AlterField(
            model_name='fittinghistory',
            name='character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='fittinghistory',
            name='fit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.fitting', verbose_name='Fitting'),
        ),
        migrations.AlterField(
            model_name='fittinghistory',
            name='implant_set',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.implantset', verbose_name='Implant Set'),
        ),
        migrations.AlterField(
            model_name='fittinghistory',
            name='logged_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Logged At'),
        ),
        migrations.AlterField(
            model_name='fleet',
            name='boss',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Fleet Boss'),
        ),
        migrations.AlterField(
            model_name='fleet',
            name='is_updating',
            field=models.BooleanField(default=False, verbose_name='Is Updating'),
        ),
        migrations.AlterField(
            model_name='fleetactivity',
            name='character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='fleetactivity',
            name='first_seen',
            field=models.DateTimeField(auto_now_add=True, verbose_name='First Seen'),
        ),
        migrations.AlterField(
            model_name='fleetactivity',
            name='fleet',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='incursions.fleet', verbose_name='Fleet'),
        ),
        migrations.AlterField(
            model_name='fleetactivity',
            name='has_left',
            field=models.BooleanField(default=False, verbose_name='Has Left'),
        ),
        migrations.AlterField(
            model_name='fleetactivity',
            name='is_boss',
            field=models.BooleanField(default=False, verbose_name='Is Boss'),
        ),
        migrations.AlterField(
            model_name='fleetactivity',
            name='last_seen',
            field=models.DateTimeField(auto_now=True, verbose_name='Last Seen'),
        ),
        migrations.AlterField(
            model_name='fleetsquad',
            name='fleet',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.fleet', verbose_name='Fleet'),
        ),
        migrations.AlterField(
            model_name='fleetsquad',
            name='squad_id',
            field=models.BigIntegerField(verbose_name='Squad ID'),
        ),
        migrations.AlterField(
            model_name='fleetsquad',
            name='wing_id',
            field=models.BigIntegerField(verbose_name='Wing ID'),
        ),
        migrations.AlterField(
            model_name='implantset',
            name='implants',
            field=models.CharField(max_length=255, unique=True, verbose_name='Implant String'),
        ),
        migrations.AlterField(
            model_name='role',
            name='description',
            field=models.TextField(blank=True, null=True, verbose_name='Role Description'),
        ),
        migrations.AlterField(
            model_name='role',
            name='name',
            field=models.CharField(max_length=64, unique=True, verbose_name='Role Name'),
        ),
        migrations.AlterField(
            model_name='role',
            name='power',
            field=models.IntegerField(default=0, help_text='Bigger is better, user can assign a power level lower than this', verbose_name='Role Power'),
        ),
        migrations.AlterField(
            model_name='skillcheck',
            name='elite',
            field=models.IntegerField(default=4, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(5)], verbose_name='Elite Level'),
        ),
        migrations.AlterField(
            model_name='skillcheck',
            name='gold',
            field=models.IntegerField(default=5, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(5)], verbose_name='Gold Level'),
        ),
        migrations.AlterField(
            model_name='skillcheck',
            name='min',
            field=models.IntegerField(default=3, validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(5)], verbose_name='Minimum Required'),
        ),
        migrations.AlterField(
            model_name='skillhistory',
            name='character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='skillhistory',
            name='logged_at',
            field=models.DateTimeField(auto_now_add=True, verbose_name='Logged At'),
        ),
        migrations.AlterField(
            model_name='skillhistory',
            name='new_level',
            field=models.IntegerField(validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(5)], verbose_name='New Level'),
        ),
        migrations.AlterField(
            model_name='skillhistory',
            name='old_level',
            field=models.IntegerField(validators=[django.core.validators.MinValueValidator(0), django.core.validators.MaxValueValidator(5)], verbose_name='Old Level'),
        ),
        migrations.AlterField(
            model_name='waitlist',
            name='is_open',
            field=models.BooleanField(default=False, verbose_name='Is Open'),
        ),
        migrations.AlterField(
            model_name='waitlist',
            name='name',
            field=models.CharField(default='Waitlist', max_length=255, verbose_name='Waitlist Name'),
        ),
        migrations.AlterField(
            model_name='waitlistcategory',
            name='name',
            field=models.CharField(max_length=255, unique=True, verbose_name='Category Name'),
        ),
        migrations.AlterField(
            model_name='waitlistentry',
            name='joined_at',
            field=models.DateTimeField(auto_now_add=True, null=True, verbose_name='Joined At'),
        ),
        migrations.AlterField(
            model_name='waitlistentry',
            name='main_character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Main Character'),
        ),
        migrations.AlterField(
            model_name='waitlistentry',
            name='waitlist',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.waitlist', verbose_name='Waitlist'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='approved',
            field=models.BooleanField(default=False, verbose_name='Approved'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='cached_time_in_fleet',
            field=models.BigIntegerField(verbose_name='Cached Time In Fleet'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='character',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='eveonline.evecharacter', verbose_name='Character'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='fit',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.fitting', verbose_name='Fitting'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='fit_analysis',
            field=models.JSONField(blank=True, null=True, verbose_name='Fit Analysis'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='implant_set',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.implantset', verbose_name='Implant Set'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='is_alt',
            field=models.BooleanField(default=False, verbose_name='Is Alt'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='messagexup',
            field=models.TextField(blank=True, null=True, verbose_name='Message XUP'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='review_comment',
            field=models.TextField(blank=True, null=True, verbose_name='Review Comment'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='tags',
            field=models.CharField(help_text='Comma Separated List', max_length=255, verbose_name='Tags'),
        ),
        migrations.AlterField(
            model_name='waitlistentryfit',
            name='waitlist_entry',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='+', to='incursions.waitlistentry', verbose_name='Waitlist Entry'),
        ),
        migrations.AlterUniqueTogether(
            name='characterbadges',
            unique_together={('character', 'badge')},
        ),
        migrations.AddIndex(
            model_name='waitlistcategory',
            index=models.Index(fields=['name'], name='incursions__name_73cfa5_idx'),
        ),
    ]



# Generated by Django 4.2.19 on 2025-04-02 13:31

import django.utils.timezone
from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('incursions', '0017_alter_incursionsconfig_status_webhooks_and_more'),
    ]

    operations = [
        migrations.AddField(
            model_name='fleet',
            name='closed',
            field=models.DateTimeField(blank=True, default=None, null=True, verbose_name='Closed'),
        ),
        migrations.AddField(
            model_name='fleet',
            name='last_updated',
            field=models.DateTimeField(auto_now=True, verbose_name='Last Updated'),
        ),
        migrations.AddField(
            model_name='fleet',
            name='open',
            field=models.BooleanField(default=False, verbose_name='Is Open'),
        ),
        migrations.AddField(
            model_name='fleet',
            name='opened',
            field=models.DateTimeField(auto_now_add=True, default=django.utils.timezone.now, verbose_name='Opened'),
            preserve_default=False,
        ),
    ]



from ninja import NinjaAPI, Schema

from allianceauth.eveonline.models import EveCharacter
from allianceauth.services.hooks import get_extension_logger

from incursions.api.schema import CharacterSchema
from incursions.providers import search_esi


class SearchResponse(Schema):
    query: str
    results: list[CharacterSchema]


class EsiSearchRequest(Schema):
    search: str
    category: str = "character"
    strict: bool = False


class EsiSearchResponse(Schema):
    character: list[int] | None = None
    corporation: list[int] | None = None
    alliance: list[int] | None = None


logger = get_extension_logger(__name__)
api = NinjaAPI()


def setup(api: NinjaAPI) -> None:
    SearchAPIEndpoints(api)


class SearchAPIEndpoints:

    tags = ["Search"]

    def __init__(self, api: NinjaAPI) -> None:

        @api.get("/search", response={200: SearchResponse, 403: dict}, tags=self.tags)
        def query(request, query: str):
            if not (request.user.has_perm("incursions.waitlist_search") or request.user.has_perm("incursions.waitlist_esi_search")):
                logger.warning(f"User {request.user} denied search for query '{query}'")
                return 403, {"error": "Permission denied"}

            characters = list(EveCharacter.objects.only("character_id", "character_name", "corporation_id", "alliance_id").filter(character_name__icontains=query))
            logger.info(f"User {request.user} performed search query '{query}' and found {len(characters)} results")
            return SearchResponse(query=query, results=characters)

        @api.post("/search", response={200: list[int], 403: dict}, tags=self.tags)
        def esi_search(request, payload: EsiSearchRequest):
            if not request.user.has_perm("incursions.waitlist_esi_search"):
                logger.warning(f"User {request.user} denied ESI search for category '{payload.category}' and search '{payload.search}'")
                return 403, {"error": "Permission denied"}

            if payload.category == "character":
                result, _ = search_esi(request.user.main_character.character_id, payload.search, payload.category, payload.strict)
                logger.info(f"User {request.user} performed ESI search for '{payload.search}' and got {len(result.get(payload.category, []))} results")
                return [x for x in result.get(payload.category, [])]

            logger.warning(f"Unsupported ESI category '{payload.category}' requested by user {request.user}")
            return 403, {"error": "Unsupported search category"}
